@@ -25,17 +25,18 @@ are always strong, and should be returned without wrapping quotes.
 
 import logging
 import mimetypes
-from typing import Optional, Iterable, Tuple, Iterator, Dict, Type
+from collections.abc import Iterable, Iterator
+from typing import Optional
 
-from .index import IndexManager
+from .index import AutoIndexManager, IndexDict, IndexKey, IndexValueIterator
 
-STORE_TYPE_ADDRESSBOOK = 'addressbook'
-STORE_TYPE_CALENDAR = 'calendar'
-STORE_TYPE_PRINCIPAL = 'principal'
-STORE_TYPE_SCHEDULE_INBOX = 'schedule-inbox'
-STORE_TYPE_SCHEDULE_OUTBOX = 'schedule-outbox'
-STORE_TYPE_SUBSCRIPTION = 'subscription'
-STORE_TYPE_OTHER = 'other'
+STORE_TYPE_ADDRESSBOOK = "addressbook"
+STORE_TYPE_CALENDAR = "calendar"
+STORE_TYPE_PRINCIPAL = "principal"
+STORE_TYPE_SCHEDULE_INBOX = "schedule-inbox"
+STORE_TYPE_SCHEDULE_OUTBOX = "schedule-outbox"
+STORE_TYPE_SUBSCRIPTION = "subscription"
+STORE_TYPE_OTHER = "other"
 VALID_STORE_TYPES = (
     STORE_TYPE_ADDRESSBOOK,
     STORE_TYPE_CALENDAR,
@@ -43,24 +44,30 @@ VALID_STORE_TYPES = (
     STORE_TYPE_SCHEDULE_INBOX,
     STORE_TYPE_SCHEDULE_OUTBOX,
     STORE_TYPE_SUBSCRIPTION,
-    STORE_TYPE_OTHER)
+    STORE_TYPE_OTHER,
+)
 
 MIMETYPES = mimetypes.MimeTypes()
-MIMETYPES.add_type('text/calendar', '.ics')  # type: ignore
-MIMETYPES.add_type('text/vcard', '.vcf')  # type: ignore
+MIMETYPES.add_type("text/calendar", ".ics")  # type: ignore
+MIMETYPES.add_type("text/vcard", ".vcf")  # type: ignore
 
-DEFAULT_MIME_TYPE = 'application/octet-stream'
-
-PARANOID = False
+DEFAULT_MIME_TYPE = "application/octet-stream"
 
 
-class File(object):
+class InvalidCTag(Exception):
+    """The request CTag can not be retrieved."""
+
+    def __init__(self, ctag) -> None:
+        self.ctag = ctag
+
+
+class File:
     """A file type handler."""
 
     content: Iterable[bytes]
     content_type: str
 
-    def __init__(self, content: Iterable[bytes], content_type: str):
+    def __init__(self, content: Iterable[bytes], content_type: str) -> None:
         self.content = content
         self.content_type = content_type
 
@@ -71,8 +78,7 @@ class File(object):
         """
 
     def normalized(self) -> Iterable[bytes]:
-        """Return a normalized version of the file.
-        """
+        """Return a normalized version of the file."""
         return self.content
 
     def describe(self, name: str) -> str:
@@ -88,18 +94,20 @@ class File(object):
         :raise NotImplementedError: If UIDs aren't supported for this format
         :raise KeyError: If there is no UID set on this file
         :raise InvalidFileContents: If the file is misformatted
-        :return: UID
+        Returns: UID
         """
         raise NotImplementedError(self.get_uid)
 
-    def describe_delta(self, name: str,
-                       previous: Optional['File']) -> Iterator[str]:
+    def describe_delta(
+            self, name: str, previous: Optional["File"]) -> Iterator[str]:
         """Describe the important difference between this and previous one.
 
-        :param name: File name
-        :param previous: Previous file to compare to.
-        :raise InvalidFileContents: If the file is misformatted
-        :return: List of strings describing change
+        Args:
+          name: File name
+          previous: Previous file to compare to.
+        Raises:
+          InvalidFileContents: If the file is misformatted
+        Returns: List of strings describing change
         """
         assert name is not None
         item_description = self.describe(name)
@@ -109,19 +117,22 @@ class File(object):
         else:
             yield "Modified " + item_description
 
-    def _get_index(self, key):
+    def _get_index(self, key: IndexKey) -> IndexValueIterator:
         """Obtain an index for this file.
 
-        :param key: Index key
-        :yield: Index values
+        Args:
+          key: Index key
+        Returns:
+          iterator over index values
         """
         raise NotImplementedError(self._get_index)
 
-    def get_indexes(self, keys):
+    def get_indexes(self, keys: Iterable[IndexKey]) -> IndexDict:
         """Obtain indexes for this file.
 
-        :param keys: Iterable of index keys
-        :return: Dictionary mapping key names to values
+        Args:
+          keys: Iterable of index keys
+        Returns: Dictionary mapping key names to values
         """
         ret = {}
         for k in keys:
@@ -129,7 +140,7 @@ class File(object):
         return ret
 
 
-class Filter(object):
+class Filter:
     """A filter that can be used to query for certain resources.
 
     Filters are often resource-type specific.
@@ -140,61 +151,70 @@ class Filter(object):
     def check(self, name: str, resource: File) -> bool:
         """Check if this filter applies to a resource.
 
-        :param name: Name of the resource
-        :param resource: File object
-        :return: boolean
+        Args:
+          name: Name of the resource
+          resource: File object
+        Returns: boolean
         """
         raise NotImplementedError(self.check)
 
-    def index_keys(self):
+    def index_keys(self) -> list[IndexKey]:
         """Returns a list of indexes that could be used to apply this filter.
 
-        :return: AND-list of OR-options
+        Returns: AND-list of OR-options
         """
         raise NotImplementedError(self.index_keys)
 
-    def check_from_indexes(self, name: str, indexes) -> bool:
+    def check_from_indexes(self, name: str, indexes: IndexDict) -> bool:
         """Check from a set of indexes whether a resource matches.
 
-        :param name: Name of the resource
-        :param indexes: Dictionary mapping index names to values
-        :return: boolean
+        Args:
+          name: Name of the resource
+          indexes: Dictionary mapping index names to values
+        Returns: boolean
         """
         raise NotImplementedError(self.check_from_indexes)
 
 
-def open_by_content_type(content: Iterable[bytes], content_type: str,
-                         extra_file_handlers) -> File:
+def open_by_content_type(
+    content: Iterable[bytes], content_type: str, extra_file_handlers
+) -> File:
     """Open a file based on content type.
 
-    :param content: list of bytestrings with content
-    :param content_type: MIME type
-    :return: File instance
+    Args:
+      content: list of bytestrings with content
+      content_type: MIME type
+    Returns: File instance
     """
-    return extra_file_handlers.get(content_type.split(';')[0], File)(
-        content, content_type)
+    return extra_file_handlers.get(content_type.split(";")[0], File)(
+        content, content_type
+    )
 
 
 def open_by_extension(
-        content: Iterable[bytes], name: str,
-        extra_file_handlers: Dict[str, Type[File]]) -> File:
+    content: Iterable[bytes],
+    name: str,
+    extra_file_handlers: dict[str, type[File]],
+) -> File:
     """Open a file based on the filename extension.
 
-    :param content: list of bytestrings with content
-    :param name: Name of file to open
-    :return: File instance
+    Args:
+      content: list of bytestrings with content
+      name: Name of file to open
+    Returns: File instance
     """
     (mime_type, _) = MIMETYPES.guess_type(name)
     if mime_type is None:
         mime_type = DEFAULT_MIME_TYPE
-    return open_by_content_type(content, mime_type,
-                                extra_file_handlers=extra_file_handlers)
+    return open_by_content_type(
+        content, mime_type, extra_file_handlers=extra_file_handlers
+    )
 
 
 class DuplicateUidError(Exception):
     """UID already exists in store."""
 
-    def __init__(self, uid: str, existing_name: str, new_name: str):
+    def __init__(self, uid: str, existing_name: str, new_name: str) -> None:
         self.uid = uid
         self.existing_name = existing_name
         self.new_name = new_name
@@ -203,14 +223,14 @@ class DuplicateUidError(Exception):
 class NoSuchItem(Exception):
     """No such item."""
 
-    def __init__(self, name: str):
+    def __init__(self, name: str) -> None:
         self.name = name
 
 
 class InvalidETag(Exception):
     """Unexpected value for etag."""
 
-    def __init__(self, name: str, expected_etag: str, got_etag: str):
+    def __init__(self, name: str, expected_etag: str, got_etag: str) -> None:
         self.name = name
         self.expected_etag = expected_etag
         self.got_etag = got_etag
@@ -219,14 +239,14 @@ class InvalidETag(Exception):
 class NotStoreError(Exception):
     """Not a store."""
 
-    def __init__(self, path: str):
+    def __init__(self, path: str) -> None:
         self.path = path
 
 
 class InvalidFileContents(Exception):
     """Invalid file contents."""
 
-    def __init__(self, content_type: str, data, error):
+    def __init__(self, content_type: str, data, error) -> None:
         self.content_type = content_type
         self.data = data
         self.error = error
@@ -235,45 +255,51 @@ class InvalidFileContents(Exception):
 class OutOfSpaceError(Exception):
     """Out of disk space."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         pass
 
 
 class LockedError(Exception):
     """File or store being accessed is locked."""
 
-    def __init__(self, path: str):
+    def __init__(self, path: str) -> None:
         self.path = path
 
 
-class Store(object):
+class Store:
     """A object store."""
 
-    extra_file_handlers: Dict[str, Type[File]]
+    extra_file_handlers: dict[str, type[File]]
 
-    def __init__(self, index):
+    def __init__(self, index, *, double_check_indexes: bool = False,
+                 index_threshold: Optional[int] = None) -> None:
         self.extra_file_handlers = {}
         self.index = index
-        self.index_manager = IndexManager(self.index)
+        self.index_manager = AutoIndexManager(
+            self.index, threshold=index_threshold)
+        self.double_check_indexes = double_check_indexes
 
-    def load_extra_file_handler(self, file_handler: Type[File]) -> None:
+    def load_extra_file_handler(self, file_handler: type[File]) -> None:
         self.extra_file_handlers[file_handler.content_type] = file_handler
 
     def iter_with_etag(
-            self, ctag: str = None) -> Iterator[Tuple[str, str, str]]:
+            self, ctag: Optional[str] = None) -> Iterator[
+                tuple[str, str, str]]:
         """Iterate over all items in the store with etag.
 
-        :param ctag: Possible ctag to iterate for
-        :yield: (name, content_type, etag) tuples
+        Args:
+          ctag: Possible ctag to iterate for
+        Returns: iterator over (name, content_type, etag) tuples
         """
         raise NotImplementedError(self.iter_with_etag)
 
     def iter_with_filter(
-            self, filter: Filter) -> Iterator[Tuple[str, File, str]]:
+            self, filter: Filter) -> Iterator[tuple[str, File, str]]:
         """Iterate over all items in the store that match a particular filter.
 
-        :param filter: Filter to apply
-        :yield: (name, file, etag) tuples
+        Args:
+          filter: Filter to apply
+        Returns: iterator over (name, file, etag) tuples
         """
         if self.index_manager is not None:
             try:
@@ -288,7 +314,8 @@ class Store(object):
         return self._iter_with_filter_naive(filter)
 
     def _iter_with_filter_naive(
-            self, filter: Filter) -> Iterator[Tuple[str, File, str]]:
+        self, filter: Filter
+    ) -> Iterator[tuple[str, File, str]]:
         for (name, content_type, etag) in self.iter_with_etag():
             if not filter.content_type == content_type:
                 continue
@@ -297,11 +324,11 @@ class Store(object):
                 if filter.check(name, file):
                     yield (name, file, etag)
             except InvalidFileContents:
-                logging.warning(
-                    'Unable to parse file %s, skipping.', name)
+                logging.warning("Unable to parse file %s, skipping.", name)
 
     def _iter_with_filter_indexes(
-            self, filter: Filter, keys) -> Iterator[Tuple[str, File, str]]:
+        self, filter: Filter, keys
+    ) -> Iterator[tuple[str, File, str]]:
         for (name, content_type, etag) in self.iter_with_etag():
             if not filter.content_type == content_type:
                 continue
@@ -314,7 +341,8 @@ class Store(object):
                     file_values = file.get_indexes(self.index.available_keys())
                 except InvalidFileContents:
                     logging.warning(
-                        'Unable to parse file %s for indexing, skipping.', name)
+                        "Unable to parse file %s for indexing, skipping.", name
+                    )
                     file_values = {}
                 self.index.add_values(name, etag, file_values)
                 if filter.check_from_indexes(name, file_values):
@@ -323,40 +351,51 @@ class Store(object):
                 if file_values is None:
                     continue
                 file = self.get_file(name, content_type, etag)
-                if PARANOID:
+                if self.double_check_indexes:
                     if file_values != file.get_indexes(keys):
-                        raise AssertionError('%r != %r' % (
-                            file_values, file.get_indexes(keys)))
-                    if (filter.check_from_indexes(name, file_values) !=
-                            filter.check(name, file)):
                         raise AssertionError(
-                            'index based filter not matching real file filter')
+                            f"{file_values!r} != {file.get_indexes(keys)!r}")
+                    if (filter.check_from_indexes(name, file_values)
+                            != filter.check(name, file)):
+                        raise AssertionError(
+                            f"index based filter {filter} "
+                            f"(values: {file_values}) not matching "
+                            "real file filter")
                 if filter.check_from_indexes(name, file_values):
                     file = self.get_file(name, content_type, etag)
                     yield (name, file, etag)
 
-    def get_file(self, name: str, content_type: Optional[str] = None,
-                 etag: Optional[str] = None) -> File:
+    def get_file(
+        self,
+        name: str,
+        content_type: Optional[str] = None,
+        etag: Optional[str] = None,
+    ) -> File:
         """Get the contents of an object.
 
-        :return: A File object
+        Returns: A File object
         """
         if content_type is None:
             return open_by_extension(
-                self._get_raw(name, etag), name,
-                extra_file_handlers=self.extra_file_handlers)
+                self._get_raw(name, etag),
+                name,
+                extra_file_handlers=self.extra_file_handlers,
+            )
         else:
             return open_by_content_type(
-                self._get_raw(name, etag), content_type,
-                extra_file_handlers=self.extra_file_handlers)
+                self._get_raw(name, etag),
+                content_type,
+                extra_file_handlers=self.extra_file_handlers,
+            )
 
     def _get_raw(
             self, name: str, etag: Optional[str] = None) -> Iterable[bytes]:
         """Get the raw contents of an object.
 
-        :param name: Filename
-        :param etag: Optional etag to return
-        :return: raw contents
+        Args:
+          name: Filename
+          etag: Optional etag to return
+        Returns: raw contents
         """
         raise NotImplementedError(self._get_raw)
 
@@ -364,79 +403,90 @@ class Store(object):
         """Return the ctag for this store."""
         raise NotImplementedError(self.get_ctag)
 
-    def import_one(self, name: str,
-                   data: Iterable[bytes],
-                   message: Optional[str] = None,
-                   author: Optional[str] = None,
-                   replace_etag: Optional[str] = None) -> Tuple[str, str]:
+    def import_one(
+        self,
+        name: str,
+        content_type: str,
+        data: Iterable[bytes],
+        message: Optional[str] = None,
+        author: Optional[str] = None,
+        replace_etag: Optional[str] = None,
+    ) -> tuple[str, str]:
         """Import a single object.
 
-        :param name: Name of the object
-        :param data: serialized object as list of bytes
-        :param message: Commit message
-        :param author: Optional author
-        :param replace_etag: Etag to replace
-        :raise NameExists: when the name already exists
-        :raise DuplicateUidError: when the uid already exists
-        :return: (name, etag)
+        Args:
+          name: Name of the object
+          content_type: Content type of the object
+          data: serialized object as list of bytes
+          message: Commit message
+          author: Optional author
+          replace_etag: Etag to replace
+        Raise:
+          NameExists: when the name already exists
+          DuplicateUidError: when the uid already exists
+        Returns: (name, etag)
         """
         raise NotImplementedError(self.import_one)
 
     def delete_one(
-            self, name: str, message: Optional[str] = None,
-            author: Optional[str] = None,
-            etag: Optional[str] = None) -> None:
+        self,
+        name: str,
+        message: Optional[str] = None,
+        author: Optional[str] = None,
+        etag: Optional[str] = None,
+    ) -> None:
         """Delete an item.
 
-        :param name: Filename to delete
-        :param message: Commit message
-        :param author: Optional author
-        :param etag: Optional mandatory etag of object to remove
-        :raise NoSuchItem: when the item doesn't exist
-        :raise InvalidETag: If the specified ETag doesn't match the current
+        Args:
+          name: Filename to delete
+          message: Commit message
+          author: Optional author
+          etag: Optional mandatory etag of object to remove
+        Raises:
+          NoSuchItem: when the item doesn't exist
+          InvalidETag: If the specified ETag doesn't match the current
         """
         raise NotImplementedError(self.delete_one)
 
     def set_type(self, store_type: str) -> None:
         """Set store type.
 
-        :param store_type: New store type (one of VALID_STORE_TYPES)
+        Args:
+          store_type: New store type (one of VALID_STORE_TYPES)
         """
         raise NotImplementedError(self.set_type)
 
     def get_type(self) -> str:
         """Get type of this store.
 
-        :return: one of VALID_STORE_TYPES
+        Returns: one of VALID_STORE_TYPES
         """
         ret = STORE_TYPE_OTHER
         for (name, content_type, etag) in self.iter_with_etag():
-            if content_type == 'text/calendar':
+            if content_type == "text/calendar":
                 ret = STORE_TYPE_CALENDAR
-            elif content_type == 'text/vcard':
+            elif content_type == "text/vcard":
                 ret = STORE_TYPE_ADDRESSBOOK
         return ret
 
     def set_description(self, description: str) -> None:
         """Set the extended description of this store.
 
-        :param description: String with description
+        Args:
+          description: String with description
         """
         raise NotImplementedError(self.set_description)
 
     def get_description(self) -> str:
-        """Get the extended description of this store.
-        """
+        """Get the extended description of this store."""
         raise NotImplementedError(self.get_description)
 
     def get_displayname(self) -> str:
-        """Get the display name of this store.
-        """
+        """Get the display name of this store."""
         raise NotImplementedError(self.get_displayname)
 
     def set_displayname(self, displayname: str) -> None:
-        """Set the display name of this store.
-        """
+        """Set the display name of this store."""
         raise NotImplementedError(self.set_displayname)
 
     def get_color(self) -> str:
@@ -448,27 +498,29 @@ class Store(object):
         raise NotImplementedError(self.set_color)
 
     def iter_changes(
-            self, old_ctag: str,
-            new_ctag: str) -> Iterator[Tuple[str, str, str, str]]:
+        self, old_ctag: str, new_ctag: str
+    ) -> Iterator[tuple[str, str, str, str]]:
         """Get changes between two versions of this store.
 
-        :param old_ctag: Old ctag (None for empty Store)
-        :param new_ctag: New ctag
-        :return: Iterator over (name, content_type, old_etag, new_etag)
+        Args:
+          old_ctag: Old ctag (None for empty Store)
+          new_ctag: New ctag
+        Returns: Iterator over (name, content_type, old_etag, new_etag)
         """
         raise NotImplementedError(self.iter_changes)
 
     def get_comment(self) -> str:
         """Retrieve store comment.
 
-        :return: Comment
+        Returns: Comment
         """
         raise NotImplementedError(self.get_comment)
 
     def set_comment(self, comment: str) -> None:
         """Set comment.
 
-        :param comment: New comment to set
+        Args:
+          comment: New comment to set
         """
         raise NotImplementedError(self.set_comment)
 
@@ -479,7 +531,7 @@ class Store(object):
     def subdirectories(self) -> Iterator[str]:
         """Returns subdirectories to probe for other stores.
 
-        :return: List of names
+        Returns: List of names
         """
         raise NotImplementedError(self.subdirectories)
 
@@ -495,9 +547,11 @@ class Store(object):
 def open_store(location: str) -> Store:
     """Open store from a location string.
 
-    :param location: Location string to open
-    :return: A `Store`
+    Args:
+      location: Location string to open
+    Returns: A `Store`
     """
     # For now, just support opening git stores
     from .git import GitStore
+
     return GitStore.open_from_path(location)
