@@ -21,11 +21,7 @@ import shutil
 import tempfile
 import unittest
 
-from xandikos.web import (
-    XandikosApp,
-    XandikosBackend,
-    WellknownRedirector,
-)
+from ..web import XandikosApp, XandikosBackend
 
 
 class WebTests(unittest.TestCase):
@@ -37,12 +33,7 @@ class WebTests(unittest.TestCase):
         path = tempfile.mkdtemp()
         try:
             backend = XandikosBackend(path)
-            backend.create_principal('foo', create_defaults=True)
-            XandikosApp(backend, 'foo')
+            backend.create_principal("foo", create_defaults=True)
+            XandikosApp(backend, "foo")
         finally:
             shutil.rmtree(path)
-
-    def test_wellknownredirector(self):
-        def app(environ, start_response):
-            pass
-        WellknownRedirector(app, '/path')
