@@ -19,27 +19,20 @@
 
 import logging
 import os
-import tempfile
 import shutil
 import stat
+import tempfile
 import unittest
 
 from dulwich.objects import Blob, Commit, Tree
 from dulwich.repo import Repo
 
-from typing import Type
+from xandikos.store import (DuplicateUidError, File, Filter, InvalidETag,
+                            NoSuchItem, Store)
 
-from xandikos.icalendar import ICalendarFile
-from xandikos.store import (
-    DuplicateUidError,
-    File,
-    InvalidETag,
-    NoSuchItem,
-    Filter,
-    Store,
-)
-from xandikos.store.git import GitStore, BareGitStore, TreeGitStore
-from xandikos.store.vdir import VdirStore
+from ..icalendar import ICalendarFile
+from ..store.git import BareGitStore, GitStore, TreeGitStore
+from ..store.vdir import VdirStore
 
 EXAMPLE_VCALENDAR1 = b"""\
 BEGIN:VCALENDAR
@@ -116,10 +109,11 @@ END:VCALENDAR
 """
 
 
-class BaseStoreTest(object):
+class BaseStoreTest:
     def test_import_one(self):
         gc = self.create_store()
-        (name, etag) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name, etag) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertIsInstance(etag, str)
         self.assertEqual(
             [("foo.ics", "text/calendar", etag)], list(gc.iter_with_etag())
@@ -127,14 +121,16 @@ class BaseStoreTest(object):
 
     def test_with_filter(self):
         gc = self.create_store()
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name2, etag2) = gc.import_one("bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name2, etag2) = gc.import_one(
+            "bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
 
         class DummyFilter(Filter):
 
             content_type = "text/calendar"
 
-            def __init__(self, text):
+            def __init__(self, text) -> None:
                 self.text = text
 
             def check(self, name, resource):
@@ -142,7 +138,7 @@ class BaseStoreTest(object):
 
         self.assertEqual(
             2,
-            len(list(gc.iter_with_filter(filter=DummyFilter(b"do something")))),
+            len(list(gc.iter_with_filter(filter=DummyFilter(b"do something"))))
         )
 
         [(ret_name, ret_file, ret_etag)] = list(
@@ -158,8 +154,10 @@ class BaseStoreTest(object):
 
     def test_get_by_index(self):
         gc = self.create_store()
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name2, etag2) = gc.import_one("bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name2, etag2) = gc.import_one(
+            "bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
         (name3, etag3) = gc.import_one(
             "bar.txt", "text/plain", [b"Not a calendar file."]
         )
@@ -171,21 +169,21 @@ class BaseStoreTest(object):
 
             content_type = "text/calendar"
 
-            def __init__(self, text):
+            def __init__(self, text) -> None:
                 self.text = text
 
             def index_keys(self):
                 return [[filtertext]]
 
             def check_from_indexes(self, name, index_values):
-                return any(self.text in v.encode() for v in index_values[filtertext])
+                return any(self.text in v for v in index_values[filtertext])
 
             def check(self, name, resource):
                 return self.text in b"".join(resource.content)
 
         self.assertEqual(
             2,
-            len(list(gc.iter_with_filter(filter=DummyFilter(b"do something")))),
+            len(list(gc.iter_with_filter(filter=DummyFilter(b"do something"))))
         )
 
         [(ret_name, ret_file, ret_etag)] = list(
@@ -211,7 +209,8 @@ class BaseStoreTest(object):
 
     def test_import_one_duplicate_uid(self):
         gc = self.create_store()
-        (name, etag) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name, etag) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertRaises(
             DuplicateUidError,
             gc.import_one,
@@ -222,11 +221,13 @@ class BaseStoreTest(object):
 
     def test_import_one_duplicate_name(self):
         gc = self.create_store()
-        (name, etag) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name, etag) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         (name, etag) = gc.import_one(
             "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR2], replace_etag=etag
         )
-        (name, etag) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name, etag) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertRaises(
             InvalidETag,
             gc.import_one,
@@ -238,8 +239,10 @@ class BaseStoreTest(object):
 
     def test_get_raw(self):
         gc = self.create_store()
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name2, etag2) = gc.import_one("bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name2, etag2) = gc.import_one(
+            "bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
         self.assertEqual(
             EXAMPLE_VCALENDAR1_NORMALIZED,
             b"".join(gc._get_raw("foo.ics", etag1)),
@@ -252,8 +255,10 @@ class BaseStoreTest(object):
 
     def test_get_file(self):
         gc = self.create_store()
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name1, etag2) = gc.import_one("bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name1, etag2) = gc.import_one(
+            "bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
         f1 = gc.get_file("foo.ics", "text/calendar", etag1)
         self.assertEqual(EXAMPLE_VCALENDAR1_NORMALIZED, b"".join(f1.content))
         self.assertEqual("text/calendar", f1.content_type)
@@ -265,7 +270,8 @@ class BaseStoreTest(object):
     def test_delete_one(self):
         gc = self.create_store()
         self.assertEqual([], list(gc.iter_with_etag()))
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertEqual(
             [("foo.ics", "text/calendar", etag1)], list(gc.iter_with_etag())
         )
@@ -275,7 +281,8 @@ class BaseStoreTest(object):
     def test_delete_one_with_etag(self):
         gc = self.create_store()
         self.assertEqual([], list(gc.iter_with_etag()))
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertEqual(
             [("foo.ics", "text/calendar", etag1)], list(gc.iter_with_etag())
         )
@@ -289,25 +296,23 @@ class BaseStoreTest(object):
     def test_delete_one_invalid_etag(self):
         gc = self.create_store()
         self.assertEqual([], list(gc.iter_with_etag()))
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name2, etag2) = gc.import_one("bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name2, etag2) = gc.import_one(
+            "bar.ics", "text/calendar", [EXAMPLE_VCALENDAR2])
         self.assertEqual(
-            set(
-                [
-                    ("foo.ics", "text/calendar", etag1),
-                    ("bar.ics", "text/calendar", etag2),
-                ]
-            ),
+            {
+                ("foo.ics", "text/calendar", etag1),
+                ("bar.ics", "text/calendar", etag2),
+            },
             set(gc.iter_with_etag()),
         )
         self.assertRaises(InvalidETag, gc.delete_one, "foo.ics", etag=etag2)
         self.assertEqual(
-            set(
-                [
-                    ("foo.ics", "text/calendar", etag1),
-                    ("bar.ics", "text/calendar", etag2),
-                ]
-            ),
+            {
+                ("foo.ics", "text/calendar", etag1),
+                ("bar.ics", "text/calendar", etag2),
+            },
             set(gc.iter_with_etag()),
         )
 
@@ -326,7 +331,7 @@ class VdirStoreTest(BaseStoreTest, unittest.TestCase):
 
 class BaseGitStoreTest(BaseStoreTest):
 
-    kls: Type[Store]
+    kls: type[Store]
 
     def create_store(self):
         raise NotImplementedError(self.create_store)
@@ -345,14 +350,16 @@ class BaseGitStoreTest(BaseStoreTest):
         logging.getLogger("").setLevel(logging.ERROR)
         gc = self.create_store()
         bid = self.add_blob(gc, "foo.ics", EXAMPLE_VCALENDAR_NO_UID)
-        self.assertEqual([("foo.ics", "text/calendar", bid)], list(gc.iter_with_etag()))
+        self.assertEqual(
+            [("foo.ics", "text/calendar", bid)], list(gc.iter_with_etag()))
         gc._scan_uids()
         logging.getLogger("").setLevel(logging.NOTSET)
 
     def test_iter_with_etag(self):
         gc = self.create_store()
         bid = self.add_blob(gc, "foo.ics", EXAMPLE_VCALENDAR1)
-        self.assertEqual([("foo.ics", "text/calendar", bid)], list(gc.iter_with_etag()))
+        self.assertEqual(
+            [("foo.ics", "text/calendar", bid)], list(gc.iter_with_etag()))
 
     def test_get_description_from_git_config(self):
         gc = self.create_store()
@@ -396,8 +403,10 @@ class BaseGitStoreTest(BaseStoreTest):
 
     def test_import_only_once(self):
         gc = self.create_store()
-        (name1, etag1) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
-        (name2, etag2) = gc.import_one("foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name1, etag1) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
+        (name2, etag2) = gc.import_one(
+            "foo.ics", "text/calendar", [EXAMPLE_VCALENDAR1])
         self.assertEqual(name1, name2)
         self.assertEqual(etag1, etag2)
         walker = gc.repo.get_walker(include=[gc.repo.refs[gc.ref]])
@@ -453,7 +462,8 @@ class BareGitStoreTest(BaseGitStoreTest, unittest.TestCase):
         gc = self.create_store()
         self.assertEqual(Tree().id.decode("ascii"), gc.get_ctag())
         self.add_blob(gc, "foo.ics", EXAMPLE_VCALENDAR1)
-        self.assertEqual(gc._get_current_tree().id.decode("ascii"), gc.get_ctag())
+        self.assertEqual(
+            gc._get_current_tree().id.decode("ascii"), gc.get_ctag())
 
 
 class TreeGitStoreTest(BaseGitStoreTest, unittest.TestCase):
